/*****************************************************************************
 * Orangutan SV-168 and LV-168 project 1:                                    *
 * Simple use of general IO to flash the LEDs and use the buzzer.            *
 *****************************************************************************/

// This program will alternately flash the red and green user LEDs until you
// ground the general-purpose user IO pin PD0 (this is the right-most of the
// eight general-purpose user IO pins at the top of the board).  When pin PDO
// is grounded, the program will pulse the buzzer pin rather than the LED pins,
// causing the buzzer to play a note.  This program does NOT use the Pololu
// AVR library (libpololu)


#include <avr/io.h>			// provides AVR port and register #defines

#define F_CPU 20000000UL	// the Orangutan xV-168 has a clock speed of 20 MHz
#include <util/delay.h>		// uses F_CPU to provide delay routines



// delay for time_ms milliseconds by looping
void delay_ms( unsigned int time_ms )
{
	unsigned int i;

	for ( i = 0; i < time_ms; i++ )
		_delay_ms( 1 );		// _delay_ms() comes from <util/delay.h>
							//  and can only delay for a max of around 13 ms
							//  when the IO clock is 20 MHz.
}


int main()
{
	// Quick AVR IO tutorial:
	//  Each AVR IO port has three registers: DDRx, PORTx, and PINx
	//  * DDRx is a Data Direction Register that controls whether an IO pin
	//    is an output or an input.  Clearing bit i of DDRx makes pin Pxi
	//    an input (this is the default state of all the IO pins).  Setting
	//    bit i of DDRx makes pin Pxi an output.
	//  * PORTx is used to control the output state of IO pins.  If pin Pxi
	//    is set as an output, setting bit i of PORTx will drive pin Pxi high
	//    while clearing bit i of PORTx will drive pin Pxi low.  If pin Pxi
	//    is an input, setting bit i of PORTx will weakly pull pin Pxi high
	//    so that it will read as high when receiving no input.  Otherwise,
	//    pin Pxi is floating (tri-stated, also known as high-Z) when set
	//    as an input.
	//  * PINx is used to read the state of an IO pin.

	// Quick C bit-logic tutorial:
	//  When working with AVR registers, you'll use the following C operators
	//  often:
	//   &  bitwise AND (this is not the same as boolean and &&)
	//   |  bitwise OR (this is not the same as boolean or ||)
	//   ^  bitwise XOR (exclusive or -- 1^1 == 0^0 == 0, 1^0 == 0^1 == 1)
	//   ~  bitwise NOT (this is not the same as boolean not !)
	//   << bitshift left  (10110010 << 2 = 11001000)
	//   >> bitshift right (10110010 >> 2 = 00101100)
	//
	//  To set a single bit i in a byte, you use:    byte = byte |  (1 << i);
	//  To clear a single bit i in a byte, you use:  byte = byte & ~(1 << i);
	//  To toggle a single bit i in a byte, you use: byte = byte ^  (1 << i);


	DDRD  &= ~(1 << PD0);	// make pin PD0 an input
	PORTD |=  (1 << PD0);	// enable pull-up on pin PD0 (this means pin PD0
							//  will read as high unless externally driven low)

	DDRD  |=  (1 << PD1);	// make red LED pin PD1 an output
	PORTD &= ~(1 << PD1);	// drive red LED pin PD1 low (turn off red LED)

	DDRD  |=  (1 << PD7);	// make green LED pin PD7 an output
	PORTD |=  (1 << PD7);   // drive green LED pin PD7 high (turn on green LED)

	DDRB  |=  (1 << PB2);	// make buzzer pin PB2 an output
	PORTB &= ~(1 << PB2);	// drive buzzer pin PB2 low
	
	
	while (1)	// loop here forever
	{
		if (PIND & (1 << PD0))	// if pin PD0 input reads as high, toggle LEDs
		{
			PORTB &= ~(1 << PB2);	// drive buzzer pin low
			delay_ms( 200 );	// loop-delay for 200 ms
			PORTD ^= (1 << PD1) | (1 << PD7);	// toggle red and green LEDs
		}
		else	// if pin PD0 is grounded, use the buzzer to make noise
		{
			delay_ms( 2 );			// loop-delay for 2 ms = buzzer frequency
									//  of 1 / .004 s = 250 Hz
			PORTB ^= (1 << PB2);	// toggle state of buzzer pin
		}
	}

	return 0;
}
